
package android.preference;

import java.text.SimpleDateFormat;
import java.util.Date;

import android.content.Context;
import android.util.AttributeSet;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TimePicker;
import android.widget.TimePicker.OnTimeChangedListener;

public class TimePickerPreference extends DialogPreference implements
        OnTimeChangedListener {

    private static final String VALIDATION_EXPRESSION = "[0-2]*[0-9]:[0-5]*[0-9]";
    private static String defaultValue = new SimpleDateFormat("HH:mm")
            .format(new Date());
    private int hour;
    private int mins;
    private TimePicker timepicker;

    public TimePickerPreference(Context context, AttributeSet attrs) {
        super(context, attrs);
        initialize();
    }

    public TimePickerPreference(Context context, AttributeSet attrs,
            int defStyle) {
        super(context, attrs, defStyle);
        initialize();
    }

    private void initialize() {
        setPersistent(true);
    }

    @Override
    protected View onCreateView(ViewGroup parent) {
        setDisplayTimeAndSave(false);
        return super.onCreateView(parent);
    }

    @Override
    protected View onCreateDialogView() {
        TimePicker timepicker = new TimePicker(getContext());
        timepicker.setIs24HourView(true);
        timepicker.setOnTimeChangedListener(this);
        int h = getHour();
        int m = getMinutes();
        if (h >= 0 && m >= 0) {
            timepicker.setCurrentHour(h);
            timepicker.setCurrentMinute(m);
        }

        this.timepicker = timepicker;
        return timepicker;
    }

    private int getMinutes() {
        String time = getPersistedString(defaultValue);
        if (time == null || !time.matches(VALIDATION_EXPRESSION)) {
            return -1;
        }

        return Integer.valueOf(time.split(":")[1]);
    }

    private int getHour() {
        String time = getPersistedString(defaultValue);
        if (time == null || !time.matches(VALIDATION_EXPRESSION)) {
            return -1;
        }

        return Integer.valueOf(time.split(":")[0]);
    }

    @Override
    public void onTimeChanged(TimePicker view, int hourOfDay, int minute) {
        updateDisplay(view, hourOfDay, minute);
    }

    private TimePicker.OnTimeChangedListener mNullTimeChangedListener = new TimePicker.OnTimeChangedListener() {
        public void onTimeChanged(TimePicker view, int hourOfDay, int minute) {

        }
    };

    private void updateDisplay(TimePicker timePicker, int hourOfDay, int minute) {
        int nextMinute = 0;
        timePicker.setOnTimeChangedListener(mNullTimeChangedListener);
        if (minute >= 55 && minute <= 59)
            nextMinute = 55;
        else if (minute >= 50)
            nextMinute = 50;
        else if (minute >= 45)
            nextMinute = 45;
        else if (minute >= 40)
            nextMinute = 40;
        else if (minute >= 35)
            nextMinute = 35;
        else if (minute >= 30)
            nextMinute = 30;
        else if (minute >= 25)
            nextMinute = 25;
        else if (minute >= 20)
            nextMinute = 20;
        else if (minute >= 15)
            nextMinute = 15;
        else if (minute >= 10)
            nextMinute = 10;
        else if (minute >= 5)
            nextMinute = 5;
        else if (minute > 0)
            nextMinute = 0;
        else {
            nextMinute = 0;
        }

        if (minute - nextMinute == 1) {
            if (minute >= 55 && minute <= 59)
                nextMinute = 00;
            else if (minute >= 50)
                nextMinute = 55;
            else if (minute >= 45)
                nextMinute = 50;
            else if (minute >= 40)
                nextMinute = 45;
            else if (minute >= 35)
                nextMinute = 40;
            else if (minute >= 30)
                nextMinute = 35;
            else if (minute >= 25)
                nextMinute = 30;
            else if (minute >= 20)
                nextMinute = 25;
            else if (minute >= 15)
                nextMinute = 20;
            else if (minute >= 10)
                nextMinute = 15;
            else if (minute >= 5)
                nextMinute = 10;
            else if (minute > 0)
                nextMinute = 5;
            else {
                nextMinute = 5;
            }
        }

        timePicker.setCurrentMinute(nextMinute);
        hour = hourOfDay;
        mins = nextMinute;
        timePicker.setOnTimeChangedListener(this);
    }

    @Override
    protected void onDialogClosed(boolean positiveResult) {
        super.onDialogClosed(positiveResult);
        timepicker.requestFocus();
        if (positiveResult) {
            if (isPersistent()) {

                setDisplayTimeAndSave(true);
            }
        }
    }

    private void setDisplayTimeAndSave(boolean useFields) {
        int h = useFields ? hour : getHour();
        int m = useFields ? mins : getMinutes();
        String summary = (h < 10 ? "0" + h : h) + ":" + (m < 10 ? "0" + m : m);
        setSummary("Aktuelle Zeit: " + summary);

        persistString(summary);

    }

    // @SuppressWarnings("static-access")
    @Override
    public void setDefaultValue(Object defaultValue) {
        super.setDefaultValue(defaultValue);
        TimePickerPreference.defaultValue = (String) defaultValue;
    }
}
