
package com.handmark.pulltorefresh.library.internal;

import android.content.Context;
import android.content.res.ColorStateList;
import android.content.res.TypedArray;
import android.text.Html;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.Interpolator;
import android.view.animation.LinearInterpolator;
import android.view.animation.RotateAnimation;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.handmark.pulltorefresh.library.PullToRefreshBase;
import com.tizianhoesch.gibmitnews.R;

public class LoadingLayout extends FrameLayout {

    static final int DEFAULT_ROTATION_ANIMATION_DURATION = 150;

    private final ImageView mHeaderImage;
    private final ProgressBar mHeaderProgress;
    private final TextView mHeaderText;
    private final TextView mSubHeaderText;

    private String mPullLabel;
    private String mRefreshingLabel;
    private String mReleaseLabel;

    private final Animation mRotateAnimation, mResetRotateAnimation;

    public LoadingLayout(Context context, final int mode, TypedArray attrs) {
        super(context);
        ViewGroup header = (ViewGroup) LayoutInflater.from(context).inflate(
                R.layout.pull_to_refresh_header, this);
        mHeaderText = (TextView) header.findViewById(R.id.pull_to_refresh_text);
        mSubHeaderText = (TextView) header.findViewById(R.id.pull_to_refresh_sub_text);
        mHeaderImage = (ImageView) header.findViewById(R.id.pull_to_refresh_image);
        mHeaderProgress = (ProgressBar) header.findViewById(R.id.pull_to_refresh_progress);

        final Interpolator interpolator = new LinearInterpolator();
        mRotateAnimation = new RotateAnimation(0, -180, Animation.RELATIVE_TO_SELF, 0.5f,
                Animation.RELATIVE_TO_SELF,
                0.5f);
        mRotateAnimation.setInterpolator(interpolator);
        mRotateAnimation.setDuration(DEFAULT_ROTATION_ANIMATION_DURATION);
        mRotateAnimation.setFillAfter(true);

        mResetRotateAnimation = new RotateAnimation(-180, 0, Animation.RELATIVE_TO_SELF, 0.5f,
                Animation.RELATIVE_TO_SELF, 0.5f);
        mResetRotateAnimation.setInterpolator(interpolator);
        mResetRotateAnimation.setDuration(DEFAULT_ROTATION_ANIMATION_DURATION);
        mResetRotateAnimation.setFillAfter(true);

        switch (mode) {
            case PullToRefreshBase.MODE_PULL_UP_TO_REFRESH:

                // Load in labels
                mHeaderImage.setImageResource(R.drawable.pulltorefresh_up_arrow);
                mPullLabel = context.getString(R.string.pull_to_refresh_from_bottom_pull_label);
                mRefreshingLabel = context
                        .getString(R.string.pull_to_refresh_from_bottom_refreshing_label);
                mReleaseLabel = context
                        .getString(R.string.pull_to_refresh_from_bottom_release_label);
                break;
            case PullToRefreshBase.MODE_PULL_DOWN_TO_REFRESH:
            default:
                mHeaderImage.setImageResource(R.drawable.pulltorefresh_down_arrow);

                // Load in labels
                mPullLabel = context.getString(R.string.pull_to_refresh_pull_label);
                mRefreshingLabel = context.getString(R.string.pull_to_refresh_refreshing_label);
                mReleaseLabel = context.getString(R.string.pull_to_refresh_release_label);
                break;
        }

        if (attrs.hasValue(R.styleable.PullToRefresh_ptrHeaderTextColor)) {
            ColorStateList colors = attrs
                    .getColorStateList(R.styleable.PullToRefresh_ptrHeaderTextColor);
            setTextColor(null != colors ? colors : ColorStateList.valueOf(0xFF000000));
        }

        reset();
    }

    public void reset() {
        mHeaderText.setText(Html.fromHtml(mPullLabel));
        mHeaderImage.setVisibility(View.VISIBLE);
        mHeaderProgress.setVisibility(View.GONE);
        if (TextUtils.isEmpty(mSubHeaderText.getText())) {
            mSubHeaderText.setVisibility(View.GONE);
        } else {
            mSubHeaderText.setVisibility(View.VISIBLE);
        }
    }

    public void releaseToRefresh() {
        mHeaderText.setText(Html.fromHtml(mReleaseLabel));
        mHeaderImage.clearAnimation();
        mHeaderImage.startAnimation(mRotateAnimation);
    }

    public void setPullLabel(String pullLabel) {
        mPullLabel = pullLabel;
    }

    public void refreshing() {
        mHeaderText.setText(Html.fromHtml(mRefreshingLabel));
        mHeaderImage.clearAnimation();
        mHeaderImage.setVisibility(View.INVISIBLE);
        mHeaderProgress.setVisibility(View.VISIBLE);
        mSubHeaderText.setVisibility(View.GONE);
    }

    public void setRefreshingLabel(String refreshingLabel) {
        mRefreshingLabel = refreshingLabel;
    }

    public void setReleaseLabel(String releaseLabel) {
        mReleaseLabel = releaseLabel;
    }

    public void pullToRefresh() {
        mHeaderText.setText(Html.fromHtml(mPullLabel));
        mHeaderImage.clearAnimation();
        mHeaderImage.startAnimation(mResetRotateAnimation);
    }

    public void setTextColor(ColorStateList color) {
        mHeaderText.setTextColor(color);
        mSubHeaderText.setTextColor(color);
    }

    public void setTextColor(int color) {
        setTextColor(ColorStateList.valueOf(color));
    }

    public void setSubHeaderText(CharSequence label) {
        if (TextUtils.isEmpty(label)) {
            mSubHeaderText.setVisibility(View.GONE);
        } else {
            mSubHeaderText.setText(label);
            mSubHeaderText.setVisibility(View.VISIBLE);
        }
    }
}
